package microsoft.dynamics.productinsights.androidsample;

import android.os.Bundle;

import androidx.appcompat.app.AppCompatActivity;

import com.microsoft.appcenter.AppCenter;
import com.microsoft.appcenter.analytics.Analytics;

import java.util.UUID;

import microsoft.dynamics.productinsights.Asset;
import microsoft.dynamics.productinsights.Product;
import microsoft.dynamics.productinsights.ProductInsightsAnalytics;
import microsoft.dynamics.productinsights.Signal;


public class MainActivity extends AppCompatActivity {

    /*****************************************************
     * Make sure to replace this with your ingestion key *
     *****************************************************/
    private String ingestionKey = "YOUR_INGESTION_KEY";
    private String appCenterSecret = "YOUR_APPCENTER_SECRET_IF_APPLICABLE";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        Thread trackSignalThread = new Thread(trackSignalRunnable);
        trackSignalThread.start();
    }

    /**
     * Put the trackSignal() in a runnable so that it can be run on a separate thread to avoid
     * hanging the UI thread.
     */
    private Runnable trackSignalRunnable = new Runnable() {
        @Override
        public void run() {
            // Use this start if your application is already using AppCenter SDK
            // alreadyUsingAppCenter();

            // Use this start if your application does not currently use AppCenter SDK
            notUsingAppCenter();

            try {
                ProductInsightsAnalytics pia = new ProductInsightsAnalytics(ingestionKey);
                trackSignal(pia);
            } catch (IllegalArgumentException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * Use this if your application already uses AppCenter SDK for telemetry.
     */
    private void alreadyUsingAppCenter() {
        AppCenter.start(
                getApplication(),
                String.format("app-secret=%s;target=%s", appCenterSecret, ingestionKey),
                Analytics.class
        );
    }

    /**
     * Use this if your application does not already use AppCenter SDK for telemetry.
     */
    private void notUsingAppCenter() {
        AppCenter.start(
                getApplication(),
                "target=" + ingestionKey,
                Analytics.class
        );
    }

    private void trackSignal(ProductInsightsAnalytics pia) {
        int count = 0;
        // Track signals continuously until user manually stops the process
        while (true) {
            Signal signal = new Signal("car_information");
            signal.setVersion("1.0.0");

            /* Set Asset related properties for the signal, all of them are optional*/
            Asset asset = new Asset();
            asset.setAssetId(generateId("Asset"));
            asset.setDeviceId(generateId("Device"));
            asset.setSerialNumber(generateId("SC3141N"));
            signal.setAsset(asset);

            /* Set Product related properties for the signal, all of them are optional */
            Product product = new Product();
            // Set the ProductID to an UUID, e.g. 457cdf42-ec53-11e9-81b4-2a2ae2dbcce4
            product.setProductId(UUID.randomUUID().toString());
            product.setModelNumber(generateId("Model"));
            product.setPartNumber(generateId("Part"));
            signal.setProduct(product);

            /* Set custom properties for the signal */
            signal.setProperty("engine_start", true);
            signal.setProperty("car_model", "Star Car");

            // This will generate a model_year between 2010 and 2019
            signal.setProperty("model_year", String.valueOf(2010 + count % 10));

            // This will generate a random integer between 0 and 10000 for RPM
            signal.setProperty("rpm", Math.round((Math.random() * 10000)));

            // This will generate a random double between 0 and 100 for temperature
            signal.setProperty("temperature", Math.random() * 100);

            // Track signal
            pia.trackSignal(signal);

            count++;
            // Sleep for 1000ms for every 20 signals sent, the sample will only stop if user manually stops it
            if (count % 20 == 0) {
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException ex) {
                    ex.printStackTrace();
                }
            }
        }
    }

    /**
     * Helper function to generate IDs in the format of prefix#### where #s are replaced with an int.
     * For example, if prefix is "Asset", a generated ID could be Asset8372.
     *
     * @param prefix
     * @return
     */
    private static String generateId(String prefix) {
        return (prefix + Math.round(Math.random() * 10000));
    }
}
